apiVersion: v1
kind: Secret
metadata:
  namespace: "{{ ns }}"
  name: "{{ name }}"
  labels:
    app.kubernetes.io/name: "{{ name | sanitize_label }}"
type: Opaque
data:
  PASSWORD: "{{ get_secret('/application', 24) | b64encode }}"
  {%- for n in range(num_values) %}
  VALUE{{ n }}: "{{ n }}"
  {%- endfor %}
