apiVersion: v1
kind: ServiceAccount
metadata:
  name: "{{ name }}"
  namespace: "{{ ns }}"
  labels:
    app.kubernetes.io/name: "{{ name }}"
automountServiceAccountToken: "{{ automount_service_account_token | default('true') }}"
