apiVersion: networking.k8s.io/v1beta1
kind: Ingress
metadata:
  name: platform
  namespace: "{{ ns }}"
  labels:
    app.kubernetes.io/name: "{{ name }}"
spec:
  rules:
  - host: "{{ domain }}"
    http:
      paths:
        - backend:
            serviceName: "{{ name }}"
            servicePort: "{{ service_port | default(80) }}"
